from django.apps import apps
from django.contrib import admin, messages
from django.utils.translation import ugettext_lazy as _
from .models import FCMDevice
from .settings import FCM_DJANGO_SETTINGS as SETTINGS
from .fcm import FCMError
User = apps.get_model(*SETTINGS["USER_MODEL"].split("."))


class DeviceAdmin(admin.ModelAdmin):
	list_display = ("__str__", "device_id", "user", "active", "date_created",)
	list_filter = ("active",)
	actions = ("send_message", "send_bulk_message", "send_data_message", "send_bulk_data_message", "enable", "disable")
	raw_id_fields = ("user",)

	if hasattr(User, "USERNAME_FIELD"):
		search_fields = ("name", "device_id", "user__%s" % (User.USERNAME_FIELD))
	else:
		search_fields = ("name", "device_id")

	def send_messages(self, request, queryset, bulk=False, data=False):
		"""
		Provides error handling for DeviceAdmin send_message and send_bulk_message methods.
		"""
		ret = []
		errors = []
		r = ""

		for device in queryset:
			try:
				if bulk:
					if data:
						r = queryset.send_message(data={"Nick" : "Mario"})
					else:
						r = queryset.send_message(title="Test notification", body="Test bulk notification")
				else:
					if data:
						r = device.send_message(data={"Nick" : "Mario"})
					else:
						r = device.send_message(title="Test notification", body="Test single notification")
				if r:
					ret.append(r)
			except FCMError as e:
				errors.append(str(e))

			if bulk:
				break

		if errors:
			self.message_user(
				request, _("Some messages could not be processed: %r" % (", ".join(errors))),
				level=messages.ERROR
			)
		if ret:
			if not bulk:
				ret = ", ".join(ret)
			if errors:
				msg = _("Some messages were sent: %s" % (ret))
			else:
				msg = _("All messages were sent: %s" % (ret))
			self.message_user(request, msg)

	def send_message(self, request, queryset):
		self.send_messages(request, queryset)

	send_message.short_description = _("Send test notification")

	def send_bulk_message(self, request, queryset):
		self.send_messages(request, queryset, True)

	send_bulk_message.short_description = _("Send test notification in bulk")

	def send_data_message(self, request, queryset):
		self.send_messages(request, queryset, False, True)

	send_data_message.short_description = _("Send test data message")

	def send_bulk_data_message(self, request, queryset):
		self.send_messages(request, queryset, True, True)

	send_bulk_data_message.short_description = _("Send test data message in bulk")

	def enable(self, request, queryset):
		queryset.update(active=True)

	enable.short_description = _("Enable selected devices")

	def disable(self, request, queryset):
		queryset.update(active=False)

	disable.short_description = _("Disable selected devices")


admin.site.register(FCMDevice, DeviceAdmin)



from django.conf import settings

if "tastypie" in settings.INSTALLED_APPS:
	# Tastypie resources are importable from the api package level (backwards compatibility)
	from .tastypie import APNSDeviceResource, GCMDeviceResource, WNSDeviceResource, APNSDeviceAuthenticatedResource, \
		GCMDeviceAuthenticatedResource, WNSDeviceAuthenticatedResource

	__all__ = [
		"APNSDeviceResource",
		"GCMDeviceResource",
		"WNSDeviceResource",
		"APNSDeviceAuthenticatedResource",
		"GCMDeviceAuthenticatedResource",
		"WNSDeviceAuthenticatedResource",
	]



from django.core.management import CommandError
from pyfcm import FCMNotification
from .settings import FCM_DJANGO_SETTINGS as SETTINGS

def fcm_send_message(registration_id,
                     title=None,
                     body=None,
                     icon=None,
                     data=None,
                     sound=None,
                     badge=None, **kwargs):
    api_key = SETTINGS.get("FCM_SERVER_KEY")
    push_service = FCMNotification(api_key=api_key)
    result = push_service.notify_single_device(registration_id=registration_id,
                                               message_title=title,
                                               message_body=body,
                                               message_icon=icon,
                                               data_message=data,
                                               sound=sound,
                                               badge=badge,
                                               **kwargs)

    # do not raise errors, pyfcm will raise exceptions if response status will
    # be anything but 200

    return result


def fcm_send_bulk_message(registration_ids,
                          title=None,
                          body=None,
                          icon=None,
                          data=None,
                          sound=None,
                          badge=None, **kwargs):
    api_key = SETTINGS.get("FCM_SERVER_KEY")
    push_service = FCMNotification(api_key=api_key)

    result = push_service.notify_multiple_devices(
        registration_ids=registration_ids,
        message_title=title,
        message_body=body,
        message_icon=icon,
        data_message=data,
        sound=sound,
        badge=badge,
        **kwargs
    )

    # do not raise errors, pyfcm will raise exceptions if response status will
    # be anything but 200

    return result

class FCMError(Exception):
    """
    PyFCM Error
    """

    pass


# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-08-05 15:40
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FCMDevice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Name')),
                ('active', models.BooleanField(default=True, help_text='Inactive devices will not be sent notifications', verbose_name='Is active')),
                ('date_created', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Creation date')),
                ('device_id', models.UUIDField(blank=True, db_index=True, help_text='GUID()', null=True, verbose_name='Device ID')),
                ('registration_id', models.TextField(verbose_name='Registration token')),
                ('type', models.CharField(choices=[('ios', 'ios'), ('android', 'android')], max_length=10)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'FCM device',
            },
        ),
    ]



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .fields import HexIntegerField
from .settings import FCM_DJANGO_SETTINGS as SETTINGS


@python_2_unicode_compatible
class Device(models.Model):
	name = models.CharField(max_length=255, verbose_name=_("Name"), blank=True, null=True)
	active = models.BooleanField(
		verbose_name=_("Is active"), default=True,
		help_text=_("Inactive devices will not be sent notifications")
	)
	user = models.ForeignKey(SETTINGS["USER_MODEL"], blank=True, null=True)
	date_created = models.DateTimeField(
		verbose_name=_("Creation date"), auto_now_add=True, null=True
	)

	class Meta:
		abstract = True

	def __str__(self):
		return (
			self.name or str(self.device_id or "") or
			"%s for %s" % (self.__class__.__name__, self.user or "unknown user")
		)


class FCMDeviceManager(models.Manager):
	def get_queryset(self):
		return FCMDeviceQuerySet(self.model)


class FCMDeviceQuerySet(models.query.QuerySet):
	def send_message(self, title=None, body=None, icon=None, data=None, sound=None, badge=None, **kwargs):
		if self:
			from .fcm import fcm_send_bulk_message

			reg_ids = list(self.filter(active=True).values_list('registration_id', flat=True))
			result = fcm_send_bulk_message(
				registration_ids=reg_ids,
				title=title,
				body=body,
				icon=icon,
				data=data,
				sound=sound,
				badge=badge,
				**kwargs
			)

			for (index, item) in enumerate(result['results']):
				if 'error' in item:
					error = item['error']

					if error == SETTINGS['ERRORS']['invalid_registration']:
						reg_id = reg_ids[index]
						device = self.filter(registration_id=reg_id).first()
						device.active = False
						device.save()

			return result


class FCMDevice(Device):
	DEVICE_TYPES = (
		(u'ios', u'ios'),
		(u'android', u'android')
	)

	device_id = models.CharField(
		verbose_name=_("Device ID"), blank=True, null=True, db_index=True,
		help_text=_("Unique device identifier"),
        max_length=150
	)
	registration_id = models.TextField(verbose_name=_("Registration token"))
	type = models.CharField(choices=DEVICE_TYPES, max_length=10)
	objects = FCMDeviceManager()

	class Meta:
		verbose_name = _("FCM device")

	def send_message(self, title=None, body=None, icon=None, data=None, sound=None, badge=None, **kwargs):
		from .fcm import fcm_send_message
		result = fcm_send_message(
			registration_id=self.registration_id,
			title=title,
			body=body,
			icon=icon,
			data=data,
			sound=None,
			badge=None,
			**kwargs
		)

		return str(result)



from __future__ import absolute_import

from fcm_django.models import FCMDevice

from tastypie.authorization import Authorization
from tastypie.authentication import BasicAuthentication
from tastypie.resources import ModelResource


class FCMDeviceResource(ModelResource):
	class Meta:
		authorization = Authorization()
		queryset = FCMDevice.objects.all()
		resource_name = "device/apns"


class APNSDeviceAuthenticatedResource(FCMDeviceResource):
	# user = ForeignKey(UserResource, "user")

	class Meta(FCMDeviceResource.Meta):
		authentication = BasicAuthentication()
	# authorization = SameUserAuthorization()

	def obj_create(self, bundle, **kwargs):
		# See https://github.com/toastdriven/django-tastypie/issues/854
		return super(APNSDeviceAuthenticatedResource, self).obj_create(bundle, user=bundle.request.user, **kwargs)



import re
import struct
from django import forms
from django.core.validators import MaxValueValidator
from django.core.validators import MinValueValidator
from django.core.validators import RegexValidator
from django.db import models, connection
from django.utils import six
from django.utils.translation import ugettext_lazy as _

UNSIGNED_64BIT_INT_MIN_VALUE = 0
UNSIGNED_64BIT_INT_MAX_VALUE = 2 ** 64 - 1

__all__ = ["HexadecimalField", "HexIntegerField"]


hex_re = re.compile(r"^(([0-9A-f])|(0x[0-9A-f]))+$")
signed_integer_engines = [
	"django.db.backends.postgresql",
	"django.db.backends.postgresql_psycopg2",
	"django.contrib.gis.db.backends.postgis",
	"django.db.backends.sqlite3"
]


def _using_signed_storage():
	return connection.settings_dict["ENGINE"] in signed_integer_engines


def _signed_to_unsigned_integer(value):
	return struct.unpack("Q", struct.pack("q", value))[0]


def _unsigned_to_signed_integer(value):
	return struct.unpack("q", struct.pack("Q", value))[0]


def _hex_string_to_unsigned_integer(value):
	return int(value, 16)


def _unsigned_integer_to_hex_string(value):
	return hex(value).rstrip("L")


class HexadecimalField(forms.CharField):
	"""
	A form field that accepts only hexadecimal numbers
	"""
	def __init__(self, *args, **kwargs):
		self.default_validators = [
			RegexValidator(hex_re, _("Enter a valid hexadecimal number"), "invalid")
		]
		super(HexadecimalField, self).__init__(*args, **kwargs)

	def prepare_value(self, value):
		# converts bigint from db to hex before it is displayed in admin
		if value and not isinstance(value, six.string_types) \
			and connection.vendor in ("mysql", "sqlite"):
			value = _unsigned_integer_to_hex_string(value)
		return super(forms.CharField, self).prepare_value(value)


class HexIntegerField(models.BigIntegerField):
	"""
	This field stores a hexadecimal *string* of up to 64 bits as an unsigned integer
	on *all* backends including postgres.

	Reasoning: Postgres only supports signed bigints. Since we don't care about
	signedness, we store it as signed, and cast it to unsigned when we deal with
	the actual value (with struct)

	On sqlite and mysql, native unsigned bigint types are used. In all cases, the
	value we deal with in python is always in hex.
	"""

	validators = [
		MinValueValidator(UNSIGNED_64BIT_INT_MIN_VALUE),
		MaxValueValidator(UNSIGNED_64BIT_INT_MAX_VALUE)
	]

	def db_type(self, connection):
		engine = connection.settings_dict["ENGINE"]
		if "mysql" in engine:
			return "bigint unsigned"
		elif "sqlite" in engine:
			return "UNSIGNED BIG INT"
		else:
			return super(HexIntegerField, self).db_type(connection=connection)

	def get_prep_value(self, value):
		""" Return the integer value to be stored from the hex string """
		if value is None or value == "":
			return None
		if isinstance(value, six.string_types):
			value = _hex_string_to_unsigned_integer(value)
		if _using_signed_storage():
			value = _unsigned_to_signed_integer(value)
		return value

	def from_db_value(self, value, expression, connection, context):
		""" Return an unsigned int representation from all db backends """
		if value is None:
			return value
		if _using_signed_storage():
			value = _signed_to_unsigned_integer(value)
		return value

	def to_python(self, value):
		""" Return a str representation of the hexadecimal """
		if isinstance(value, six.string_types):
			return value
		if value is None:
			return value
		return _unsigned_integer_to_hex_string(value)

	def formfield(self, **kwargs):
		defaults = {"form_class": HexadecimalField}
		defaults.update(kwargs)
		# yes, that super call is right
		return super(models.IntegerField, self).formfield(**defaults)

	def run_validators(self, value):
		# make sure validation is performed on integer value not string value
		value = _hex_string_to_unsigned_integer(value)
		return super(models.BigIntegerField, self).run_validators(value)



from django.conf import settings

FCM_DJANGO_SETTINGS = getattr(settings, "FCM_DJANGO_SETTINGS", {})

# FCM
FCM_DJANGO_SETTINGS.setdefault("FCM_SERVER", "https://fcm.googleapis.com/fcm/send")
FCM_DJANGO_SETTINGS.setdefault("FCM_SERVER_KEY", None)

# User model
FCM_DJANGO_SETTINGS.setdefault("USER_MODEL", settings.AUTH_USER_MODEL)

FCM_DJANGO_SETTINGS.setdefault("ERRORS", {
    'invalid_registration': 'InvalidRegistration',
    'missing_registration': 'MissingRegistration',
    'not_registered': 'NotRegistered',
    'invalid_package_name': 'InvalidPackageName'
})


# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-08-08 15:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fcm_django', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fcmdevice',
            name='device_id',
            field=models.CharField(blank=True, db_index=True, help_text='Unique device identifier', max_length=150, null=True, verbose_name='Device ID'),
        ),
    ]



from __future__ import absolute_import
from rest_framework import permissions
from rest_framework.serializers import ModelSerializer, ValidationError, Serializer, CurrentUserDefault
from rest_framework.viewsets import ModelViewSet
from fcm_django.models import FCMDevice

# Fields


# Serializers
class DeviceSerializerMixin(ModelSerializer):
	class Meta:
		fields = ("id", "name", "registration_id", "device_id", "active", "date_created", "type")
		read_only_fields = ("date_created",)

		# See https://github.com/tomchristie/django-rest-framework/issues/1101
		extra_kwargs = {"active": {"default": True}}

class UniqueRegistrationSerializerMixin(Serializer):
	def validate(self, attrs):
		devices = None
		primary_key = None
		request_method = None

		if self.initial_data.get("registration_id", None):
			if self.instance:
				request_method = "update"
				primary_key = self.instance.id
			else:
				request_method = "create"
		else:
			if self.context["request"].method in ["PUT", "PATCH"]:
				request_method = "update"
				primary_key = attrs["id"]
			elif self.context["request"].method == "POST":
				request_method = "create"

		Device = self.Meta.model
		if request_method == "update":
			devices = Device.objects.filter(registration_id=attrs["registration_id"]) \
				.exclude(id=primary_key)
		elif request_method == "create":
			# if request authenticated, unique together with registration_id and user
			user = self.context['request'].user
			if user is not None:
				devices = Device.objects.filter(registration_id=attrs["registration_id"], user=user)
			else:
				devices = Device.objects.filter(registration_id=attrs["registration_id"])

		if devices:
			raise ValidationError({'registration_id': 'This field must be unique.'})
		return attrs

class FCMDeviceSerializer(ModelSerializer, UniqueRegistrationSerializerMixin):

	class Meta(DeviceSerializerMixin.Meta):
		model = FCMDevice

		extra_kwargs = {"id": {"read_only": False, "required": False}}


# Permissions
class IsOwner(permissions.BasePermission):
	def has_object_permission(self, request, view, obj):
		# must be the owner to view the object
		return obj.user == request.user


# Mixins
class DeviceViewSetMixin(object):
	lookup_field = "registration_id"

	def perform_create(self, serializer):
		if self.request.user.is_authenticated():
			serializer.save(user=self.request.user)
		return super(DeviceViewSetMixin, self).perform_create(serializer)

	def perform_update(self, serializer):
		if self.request.user.is_authenticated():
			serializer.save(user=self.request.user)
		return super(DeviceViewSetMixin, self).perform_update(serializer)


class AuthorizedMixin(object):
	permission_classes = (permissions.IsAuthenticated, IsOwner)

	def get_queryset(self):
		# filter all devices to only those belonging to the current user
		return self.queryset.filter(user=self.request.user)


# ViewSets
class FCMDeviceViewSet(DeviceViewSetMixin, ModelViewSet):
	queryset = FCMDevice.objects.all()
	serializer_class = FCMDeviceSerializer


class FCMDeviceAuthorizedViewSet(AuthorizedMixin, FCMDeviceViewSet):
	pass


#!/usr/bin/env python
import os.path
import fcm_django
from distutils.core import setup
import setuptools

VERSION = '0.2.0'

CLASSIFIERS = [
	"Development Status :: 5 - Production/Stable",
	"Environment :: Web Environment",
	"Framework :: Django",
	"Intended Audience :: Developers",
	"License :: OSI Approved :: MIT License",
	"Programming Language :: Python",
	"Programming Language :: Python :: 2.7",
	"Programming Language :: Python :: 3",
	"Programming Language :: Python :: 3.4",
	"Programming Language :: Python :: 3.5",
	"Topic :: Software Development :: Libraries :: Python Modules",
	"Topic :: System :: Networking",
]

setup(
	name="fcm-django",
	packages=[
		"fcm_django",
		"fcm_django/api",
		"fcm_django/migrations",
		"fcm_django/management",
		"fcm_django/management/commands",
	],
    install_requires=[
        'pyfcm==1.0.1',
        'Django'
    ],
	author=fcm_django.__author__,
	author_email=fcm_django.__email__,
	classifiers=CLASSIFIERS,
	description="Send push notifications to mobile devices through FCM in Django.",
	download_url="https://github.com/xtrinch/fcm-django/tarball/master",
	long_description='',
	url="https://github.com/xtrinch/fcm-django",
	version=fcm_django.__version__,
)
